MAIN_CONTROLLER: str = "__main__"

WHEEL_SPEED_CONTROLLER: str = "wsc"
LEFT_FRONT_WHEEL_SPEED_SENSOR: str = "wss_lf"
RIGHT_FRONT_WHEEL_SPEED_SENSOR: str = "wss_rf"
CENTER_REAR_WHEEL_SPEED_SENSOR: str = "wss_cr"
LEFT_REAR_WHEEL_SPEED_SENSOR: str = "wss_lr"
RIGHT_REAR_WHEEL_SPEED_SENSOR: str = "wss_rr"
ODOMETER: str = "odm"
POWER_CONTROLLER: str = "pc"
THROTTLE_PEDAL: str = "tpd"
BRAKE_PEDAL: str = "bpd"
DC_MOTOR_CONTROLLER_A: str = "dmca"
DC_MOTOR_CONTROLLER_B: str = "dmcb"
GPS_RECEIVER: str = "gps"
LEFT_INDICATOR: str = "lindi"
RIGHT_INDICATOR: str = "rindi"
